using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Text;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.Utilities;

namespace Vevo.Shared.Domain.Logging
{
    public class ExceptionLogPurger
    {
        #region Private

        private const int ExpiredLogDays = 10;
        private const int FileNamePrefixLength = 4;     // for "log_"
        private const int MinimumPurgeIntervalHours = 4;

        private static TimeSpan _minimumPurgeInterval = TimeSpan.FromHours( MinimumPurgeIntervalHours );
        private static DateTime? _lastPurgeTime = null;
        private static CultureInfo _usCultureInfo = CultureInfo.GetCultureInfo( "en-US" );

        private IFileManager _fileManager;


        private bool IsExpired( string filePath )
        {
            string fileName = Path.GetFileNameWithoutExtension( filePath );

            Assume.IsTrue( fileName.ToLower().StartsWith( "log_" ) );

            if (!IsValidFileName( fileName ))
                return false;

            DateTime fileDate = GetDateFromFileName( fileName );
            return fileDate.AddDays( ExpiredLogDays ) <= GetCurrentDateNoTime();
        }

        private bool IsValidFileName( string fileName )
        {
            string[] splited = GetDateElementArray( fileName );
            if (splited.Length != 3)
                return false;

            // Make sure all date elements are numbers
            foreach (string value in splited)
            {
                if (ConvertUtilities.ToInt32( value ) == 0)
                    return false;
            }

            // Verify date validity (e.g. not 13/32/2010)
            // Put date in mm/dd/yyyy format and try if parsing succeeds.
            string dateString = String.Format( "{0}/{1}/{2}", splited[1], splited[2], splited[0] );

            DateTime dateTime;
            return DateTime.TryParse(
                dateString, _usCultureInfo.DateTimeFormat, DateTimeStyles.None, out dateTime );
        }

        private string[] GetDateElementArray( string fileName )
        {
            string dateString = fileName.Substring( FileNamePrefixLength );        // e.g. "2010-05-10"
            return dateString.Split( '-' );
        }

        private DateTime GetCurrentDateNoTime()
        {
            // Get current date with no time element
            DateTime now = DateTime.Now;
            return new DateTime( now.Year, now.Month, now.Day );
        }

        private DateTime GetDateFromFileName( string fileName )
        {
            string[] splited = GetDateElementArray( fileName );

            return new DateTime(
                ConvertUtilities.ToInt32( splited[0] ),         // Year
                ConvertUtilities.ToInt32( splited[1] ),         // Month
                ConvertUtilities.ToInt32( splited[2] ) );       // Day
        }

        private bool IsIntervalReached()
        {
            if (_lastPurgeTime == null)
                return true;

            return _lastPurgeTime.Value.Add( MinimumPurgeInterval ) <= DateTime.Now;
        }

        private void SetLastPurgeTime()
        {
            _lastPurgeTime = DateTime.Now;
        }

        #endregion


        #region Constructors

        public ExceptionLogPurger( IFileManager fileManager )
        {
            _fileManager = fileManager;
        }

        #endregion


        #region Public Properties

        public static TimeSpan MinimumPurgeInterval
        {
            get
            {
                return _minimumPurgeInterval;
            }
            set
            {
                _minimumPurgeInterval = value;
            }
        }

        #endregion


        #region Public Methods

        public static void ResetLastPurgeTime()
        {
            _lastPurgeTime = null;
        }

        public void PurgeExpiredLogs()
        {
            try
            {
                if (!IsIntervalReached())
                    return;

                SetLastPurgeTime();

                string[] filePaths = _fileManager.SearchFiles( ExceptionLog.LogFilePath, "log_*.txt" );

                foreach (string filePath in filePaths)
                {
                    if (IsExpired( filePath ))
                        _fileManager.Delete( filePath );
                }
            }
            catch
            {
                // Never return an exception for this function
            }
        }

        #endregion
    }
}
