using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Shared.DataAccess;

namespace Vevo.DataAccessLib
{
    public static class LanguageTextAccess
    {
        public const string GlobalID = "0";


        #region Private
        private static string CreateSortByText( string sortBy )
        {
            string text;

            if (sortBy == null || sortBy.Trim() == "")
                text = "";
            else
                text = "ORDER BY " + sortBy + " ";

            text = text.ToLower();
            text = text.Replace( " pageid", " LanguageText.PageID" );
            text = text.Replace( " cultureid", " LanguageText.CultureID" );
            text = text.Replace( "languagename", "DisplayName" );

            return text;
        }

        private static bool IsSameKey(
            string key1, string key2,
            string newKey1, string newKey2 )
        {
            if (String.Compare( key1, newKey1, true ) == 0 &&
                String.Compare( key2, newKey2, true ) == 0)
                return true;
            else
                return false;
        }

        private static int UpdateImplementation(
            string pageID,
            string cultureID,
            string keyName,
            string newCultureID,
            string newKeyName,
            string newTextData )
        {
            if (!IsSameKey( cultureID, keyName, newCultureID, newKeyName ) &&
                IsExistingKeyword( pageID, newCultureID, newKeyName ))
            {
                throw new DuplicatedPrimaryKeyException(
                    "Table: LanguageText, PageID: " + pageID +
                    ", CultureID: " + newCultureID + ", KeyName: " + newKeyName );
            }

            return DataAccess.ExecuteNonQuery(
                 "UPDATE LanguageText " +
                 "SET CultureID = @NewCultureID, KeyName = @NewKeyName, TextData = @NewTextData " +
                 "WHERE PageID = @PageID AND CultureID = @CultureID AND KeyName = @KeyName; ",
                 DataAccess.CreateParameterString( newCultureID ),
                 DataAccess.CreateParameterString( newKeyName ),
                 DataAccess.CreateParameterString( newTextData ),
                 DataAccess.CreateParameterString( pageID ),
                 DataAccess.CreateParameterString( cultureID ),
                 DataAccess.CreateParameterString( keyName ) );
        }
        #endregion


        public static void Create(
            string pageID,
            string cultureID,
            string keyName,
            string textData )
        {
            if (IsExistingKeyword( pageID, cultureID, keyName ))
                throw new DuplicatedPrimaryKeyException(
                    "Table: LanguageText, PageID: " + pageID +
                    ", CultureID: " + cultureID + ", KeyName: " + keyName );

            DataAccess.ExecuteNonQuery(
                "INSERT INTO LanguageText " +
                "    (PageID, CultureID, KeyName, TextData) " +
                "VALUES " +
                "    (@PageID, @CultureID, @KeyName, @TextData); ",
                DataAccess.CreateParameterString( pageID ),
                DataAccess.CreateParameterString( cultureID ),
                DataAccess.CreateParameterString( keyName ),
                DataAccess.CreateParameterString( textData ) );
        }

        public static void CreateGlobal(
            string cultureID,
            string keyName,
            string textData )
        {
            Create( GlobalID, cultureID, keyName, textData );
        }

        public static void Update(
            string pageID,
            string cultureID,
            string keyName,
            string newCultureID,
            string newKeyName,
            string newTextData )
        {
            UpdateImplementation( pageID, cultureID, keyName, newCultureID, newKeyName, newTextData );
        }

        public static void CreateOrUpdate(
            string pageID,
            string cultureID,
            string keyName,
            string newCultureID,
            string newKeyName,
            string newTextData )
        {
            if (UpdateImplementation( pageID, cultureID, keyName, newCultureID, newKeyName, newTextData ) == 0)
            {
                Create( pageID, newCultureID, newKeyName, newTextData );
            }
        }

        public static void Delete(
            string pageID,
            string cultureID,
            string keyName )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM LanguageText " +
                "WHERE PageID = @PageID AND CultureID = @CultureID AND KeyName = @KeyName; ",
                DataAccess.CreateParameterString( pageID ),
                DataAccess.CreateParameterString( cultureID ),
                DataAccess.CreateParameterString( keyName ) );
        }

        public static void DeleteByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM LanguageText " +
                "WHERE CultureID = @CultureID;",
                DataAccess.CreateParameterString( cultureID )
            );

        }

        public static void DeleteByPageID( string pageID )
        {
            DataAccess.ExecuteNonQuery(
               "DELETE " +
               "FROM LanguageText " +
               "WHERE PageID = @PageID; ",
               DataAccess.CreateParameterString( pageID ) );
        }

        public static string GetTextData( string pageID, string cultureID, string keyName )
        {
            string textData = DataAccess.ExecuteScalar(
                "SELECT TextData " +
                "FROM LanguageText " +
                "WHERE PageID = @PageID AND CultureID = @CultureID AND KeyName = @KeyName; ",
                DataAccess.CreateParameterString( pageID ),
                DataAccess.CreateParameterString( cultureID ),
                DataAccess.CreateParameterString( keyName ) );
            return textData;
        }

        public static DataTable GetTextByPage( string pageID, string cultureID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT PageID, CultureID, KeyName, TextData " +
                "FROM LanguageText " +
                "WHERE (PageID = @PageID OR PageID = @GlobalID) " +
                "    AND CultureID = @CultureID; ",
                DataAccess.CreateParameterString( pageID ),
                DataAccess.CreateParameterString( GlobalID ),
                DataAccess.CreateParameterString( cultureID ) );

            DataColumn[] keys = new DataColumn[] { 
                table.Columns["PageID"], table.Columns["CultureID"], table.Columns["KeyName"] };
            table.PrimaryKey = keys;
            return table;
        }

        public static DataTable GetTextByKeyname( string pageID, string KeyName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT PageID, CultureID, KeyName, TextData " +
                "FROM LanguageText " +
                "WHERE (PageID = @PageID OR PageID = @GlobalID) " +
                "    AND KeyName = @KeyName; ",
                DataAccess.CreateParameterString( pageID ),
                DataAccess.CreateParameterString( GlobalID ),
                DataAccess.CreateParameterString( KeyName ) );

            DataColumn[] keys = new DataColumn[] { 
                table.Columns["PageID"], table.Columns["CultureID"], table.Columns["KeyName"] };
            table.PrimaryKey = keys;
            return table;
        }

        public static DataTable GetTextByPageAll( string pageID, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT PageID, Culture.CultureID, DisplayName AS LanguageName, KeyName, TextData " +
                "FROM Culture INNER JOIN LanguageText " +
                "ON Culture.CultureID = LanguageText.CultureID " +
                "WHERE PageID = @PageID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( pageID ) );

            DataColumn[] keys = new DataColumn[] {
                table.Columns["PageID"], table.Columns["CultureID"], table.Columns["KeyName"] };
            table.PrimaryKey = keys;
            return table;
        }

        public static DataTable GetByCultureID( string cultureID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT PageID, CultureID, KeyName, TextData " +
                "FROM LanguageText " +
                "WHERE CultureID = @CultureID; ",
                DataAccess.CreateParameterString( cultureID ) );

            DataColumn[] keys = new DataColumn[] { 
                table.Columns["PageID"], table.Columns["CultureID"], table.Columns["KeyName"] };
            table.PrimaryKey = keys;
            return table;
        }

        public static void CreateByCultureID( string cultureID, string cultureIDBaseOn )
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO LanguageText (PageID, CultureID, KeyName, TextData) " +
                "SELECT PageID,'" + cultureID + "',KeyName,TextData FROM LanguageText WHERE " +
                "CultureID = @cultureID; ",
                DataAccess.CreateParameterString( cultureIDBaseOn ) );
        }

        public static bool IsExistingKeyword(
            string pageID,
            string cultureID,
            string keyName )
        {
            string count = DataAccess.ExecuteScalar(
                "SELECT COUNT(*) " +
                "FROM LanguageText " +
                "WHERE PageID = @PageID AND CultureID = @CultureID AND KeyName = @KeyName; ",
                DataAccess.CreateParameterString( pageID ),
                DataAccess.CreateParameterString( cultureID ),
                DataAccess.CreateParameterString( keyName ) );

            return (int.Parse( count ) > 0);
        }

        public static DataTable Search(
            string cultureID,
            string sortBy,
            string fieldName,
            string val,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            string whereCommand = SearchFilterHelper.CreateCommandWithWhere( 
                SearchFilterType.Text, fieldName, val, String.Empty, parameterList );

            string cultureCommand = String.Empty;

            if (!String.IsNullOrEmpty( cultureID ))
            {
                cultureCommand = " AND Culture.CultureID = @CultureID ";
                parameterList.Add( DataAccess.CreateParameterString( cultureID ) );
            }

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT LanguageText.PageID, Path, Culture.CultureID, DisplayName AS LanguageName, KeyName, TextData " +
                "FROM (Culture INNER JOIN LanguageText " +
                "ON Culture.CultureID = LanguageText.CultureID) INNER JOIN Page ON LanguageText.PageID = Page.PageID " +
                whereCommand +
                cultureCommand +
                CreateSortByText( sortBy ) + ";",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = table.Rows.Count;
            return DataAccess.CopyTableRows( table, startIndex, endIndex );
        }
    }
}
