using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Payments;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class GiftCertificateRepository : IGiftCertificateRepository
    {
        #region Private

        private const string RepositoryName = "GiftCertificateRepository";
        private const string GetAllSignature = "GetAll";
        private const string GetOneSignature = "GetOne";
        private const string GetCodeFromUrlParameterSignature = "GetCodeFromUrlParameter";
        private const string GetCodeFromOrderItemIDSignature = "GetCodeFromOrderItemID";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string SearchGiftCertificateSignature = "SearchGiftCertificate";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency 
            = new CacheDependencyUtility( new string[] { "Orders", "OrderItem", "GiftCertificate" } );

        private GiftCertificateRepositoryNoCache _repository = new GiftCertificateRepositoryNoCache();

        #endregion

        #region Constructors

        public GiftCertificateRepository()
        {
        }

        public GiftCertificateRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public GiftCertificate Create( GiftCertificate giftCertificate )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            GiftCertificate newGiftCertificate = _repository.Create( giftCertificate );
            _dataCache.ClearRepositoryCache();
            return newGiftCertificate;
        }

        public GiftCertificate Update( GiftCertificate giftCertificate )
        {
            GiftCertificate result = _repository.Update( giftCertificate );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public IList<GiftCertificate> GetAllByOrderID( string orderID )
        {
            IList<GiftCertificate> giftList = (IList<GiftCertificate>) _dataCache.GetValue( GetAllSignature, _cacheDependency, orderID );
            if (giftList == null)
            {
                giftList = _repository.GetAllByOrderID( orderID );
                _dataCache.SetValue( GetAllSignature, giftList, orderID );
            }
            return giftList;
        }

        public GiftCertificate GetOne( string giftCertificateCode )
        {
            GiftCertificate result = (GiftCertificate) _dataCache.GetValue( GetOneSignature, _cacheDependency, giftCertificateCode );

            if (result == null)
            {
                result = _repository.GetOne( giftCertificateCode );
                _dataCache.SetValue( GetOneSignature, result, giftCertificateCode );
            }

            return result;
        }

        public string GetCodeFromUrlParameter( string urlParameter )
        {
            string result = (string) _dataCache.GetValue( GetCodeFromUrlParameterSignature, _cacheDependency, urlParameter );

            if (result == null)
            {
                result = _repository.GetCodeFromUrlParameter( urlParameter );
                _dataCache.SetValue( GetCodeFromUrlParameterSignature, result, urlParameter );
            }

            return result;
        }

        public string GetCodeFromOrderItemID( string orderItemID )
        {
            string result = (string) _dataCache.GetValue( GetCodeFromOrderItemIDSignature, _cacheDependency, orderItemID );

            if (result == null)
            {
                result = _repository.GetCodeFromOrderItemID( orderItemID );
                _dataCache.SetValue( GetCodeFromOrderItemIDSignature, result, orderItemID );
            }

            return result;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );
            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }
            return list;
        }

        public void Delete( string giftCertificateCode )
        {
            _repository.Delete( giftCertificateCode );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteByOrderID( string orderID )
        {
            _repository.DeleteByOrderID( orderID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteByOrderItemID( string orderItemID )
        {
            _repository.DeleteByOrderItemID( orderItemID );
            _dataCache.ClearRepositoryCache();
        }

        public DataTable SearchGiftCertificate(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchGiftCertificateSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchGiftCertificate( sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchGiftCertificateSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (DataTable) result.First;
        }

        #endregion
    }
}
