﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Returns;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class RmaActionRepositoryNoCache : IRmaActionRepository
    {
        #region Private

        private const string _allField = "RmaActionID, Name, IsEnabled ";

        private string CreateSortByText( string sortBy )
        {
            if ( String.IsNullOrEmpty( sortBy ) )
                return String.Empty;
            else
                return "ORDER BY " + sortBy;
        }

        private RmaAction Create( RmaAction rmaAction )
        {
            string rmaActionID = DataAccess.ExecuteScalar( "INSERT INTO RmaAction( Name, IsEnabled ) " +
                "VALUES ( @Name, @IsEnabled ); SELECT @@Identity; ",
                DataAccess.CreateParameterString( rmaAction.Name ),
                DataAccess.CreateParameterBool( rmaAction.IsEnabled ) );

            rmaAction.RmaActionID = rmaActionID;

            return rmaAction;
        }

        private RmaAction Update( RmaAction rmaAction )
        {
            string sql = "UPDATE RmaAction SET Name = @Name, IsEnabled = @IsEnabled WHERE RmaActionID = @RmaActionID; ";

            DataAccess.ExecuteNonQuery( sql,
                DataAccess.CreateParameterString( rmaAction.Name ),
                DataAccess.CreateParameterBool( rmaAction.IsEnabled ),
                DataAccess.CreateParameterString( rmaAction.RmaActionID ) );

            return rmaAction;
        }

        private RmaAction CreateOneFromRow( DataRow row )
        {
            RmaAction rmaAction = new RmaAction();
            rmaAction.RmaActionID = row[ "RmaActionID" ].ToString();
            rmaAction.Name = row[ "Name" ].ToString();
            rmaAction.IsEnabled = ConvertUtilities.ToBoolean( row[ "IsEnabled" ] );

            return rmaAction;
        }

        private IList<RmaAction> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<RmaAction> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<RmaAction> list = new List<RmaAction>();

            for ( int i = startIndex; i <= endIndex && i < table.Rows.Count; i++ )
                list.Add( CreateOneFromRow( table.Rows[ i ] ) );

            return list;
        }

        private IList<RmaAction> Search( string sortBy, SearchFilter searchFilter )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, searchFilter.FieldName, parameterList );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _allField +
                "FROM RmaAction " + 
                commandSearch + 
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            DataColumn[] keys = new DataColumn[] { table.Columns[ "RmaActionID" ] };
            table.PrimaryKey = keys;

            return ConvertTableToList( table );
        }

        private IList<RmaAction> GetAllRmaAction( BoolFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _allField + "FROM RmaAction " +
                FlagFilterHelper.CreateFlagFilterWithWhere( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                "; " ); 

            DataColumn[] keys = new DataColumn[ 1 ] { table.Columns[ "NewsID" ] };
            table.PrimaryKey = keys;

            return ConvertTableToList( table );
        }

        #endregion

        #region Public Methods

        public RmaAction Save( RmaAction rmaAction )
        {
            RmaAction result;

            if ( rmaAction.RmaActionID == "0" )
                result = Create( rmaAction );
            else
                result = Update( rmaAction );

            return result;
        }

        public RmaAction GetOne( string rmaActionID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _allField +
                "FROM RmaAction " +
                "WHERE RmaActionID = @RmaActionID; ",
                DataAccess.CreateParameterString( rmaActionID ) );

            if ( table.Rows.Count > 0 )
                return CreateOneFromRow( table.Rows[ 0 ] );
            else
                return RmaAction.Null;
        }

        public IList<RmaAction> GetAll( BoolFilter isEnabled )
        {
            return GetAllRmaAction( isEnabled );
        }

        public void Delete( string rmaActionID )
        {
            using ( DataTransactionScope transaction = new DataTransactionScope() )
            {
                DataAccess.ExecuteNonQuery( "DELETE FROM RmaAction WHERE RmaActionID = @RmaActionID; ",
                DataAccess.CreateParameterString( rmaActionID ) );

                transaction.Complete();
            }
        }

        public IList<RmaAction> SearchRmaAction( string sortBy, SearchFilter searchFilter )
        {
            return Search( sortBy, searchFilter );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                 "SELECT " + _allField +
                 "FROM RmaAction " +
                 "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
