using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml.Serialization;
using Iesi.Collections;

namespace Vevo.Domain.Payments
{
    //-----------------------------------------------------------------------
    // GatewayCurrencyList
    //
    // For XML serialization.
    //-----------------------------------------------------------------------
    #region XmlSerialization
    [Serializable]
    [XmlRoot( "GatewayCurrencyList" )]
    public class GatewayCurrencyList
    {
        [XmlArrayItem( "Currency" )]
        public PaymentCurrency[] Currencies;
    }

    [Serializable]
    public class PaymentCurrency
    {
        private string _name;
        private string _currencyCode;
        private string _displayName;


        public string Name { get { return _name; } set { _name = value; } }
        public string CurrencyCode { get { return _currencyCode; } set { _currencyCode = value; } }
        public string DisplayName { get { return _displayName; } set { _displayName = value; } }
    }
    #endregion


    //-----------------------------------------------------------------------
    // GatewayCurrency
    //-----------------------------------------------------------------------
    [Serializable]
    public class GatewayCurrency
    {
        [Serializable]
        private struct GatewayDetails
        {
            public string PaymentName;
            public string[] CurrencyCodes;


            public GatewayDetails( string c_paymentName, string[] c_currencyCodes )
            {
                PaymentName = c_paymentName;
                CurrencyCodes = c_currencyCodes;
            }
        }


        public GatewayCurrency( string gatewayCurrencyFile )
        {
            SetUpCurrencyList( gatewayCurrencyFile );
        }


        #region Private
        private GatewayCurrencyList _currencyList;
        private ArrayList _gatewayList = new ArrayList();


        private void SetUpCurrencyList( string gatewayCurrencyFile )
        {
            using (FileStream fs = new FileStream( gatewayCurrencyFile, FileMode.Open, FileAccess.Read ))
            {
                XmlSerializer serializer = new XmlSerializer( typeof( GatewayCurrencyList ) );
                _currencyList = (GatewayCurrencyList) serializer.Deserialize( fs );
            }

            foreach (PaymentCurrency currency in _currencyList.Currencies)
            {
                currency.DisplayName = currency.CurrencyCode + " (" + currency.Name + ")";
            }
        }

        private PaymentCurrency GetCurrencyDetails( string currencyCode )
        {
            foreach (PaymentCurrency currency in _currencyList.Currencies)
            {
                if (String.Compare( currency.CurrencyCode, currencyCode, true ) == 0)
                    return currency;
            }

            throw new Exception( "Unknown gateway currency code: '" + currencyCode + "'" );
        }

        private Set GetIntersectionOfGatewayCurrencies()
        {
            Set validCurrencies = null;

            if (_gatewayList.Count > 0)
            {
                for (int i = 0; i < _gatewayList.Count; i++)
                {
                    GatewayDetails gateway = (GatewayDetails) _gatewayList[i];
                    Set currencies = new SortedSet( gateway.CurrencyCodes );

                    if (validCurrencies == null)
                        validCurrencies = currencies;
                    else
                        validCurrencies = validCurrencies & currencies;     // Set intersection
                }
            }

            if (validCurrencies == null)
                validCurrencies = new SortedSet();

            return validCurrencies;
        }

        private PaymentCurrency[] GetCurrencyDetailsForSet( Set currencies )
        {
            PaymentCurrency[] result = new PaymentCurrency[currencies.Count];

            int i = 0;
            foreach (Object obj in currencies)
            {
                string currencyCode = (string) obj;
                result[i] = GetCurrencyDetails( currencyCode );
                i++;
            }

            return result;
        }
        #endregion


        public void AddGateway( string paymentName, string[] currencyCodes )
        {
            _gatewayList.Add( new GatewayDetails( paymentName, currencyCodes ) );
        }

        public PaymentCurrency[] GetValidCurrencies()
        {
            Set validCurrencies = GetIntersectionOfGatewayCurrencies();

            return GetCurrencyDetailsForSet( validCurrencies );
        }

    }
}
