using System;
using System.Collections;
using System.Data;
using System.Text;
using Vevo.Shared.DataAccess;

namespace Vevo.Reports
{
    public class BestSellingReportBuilder
    {
        private string GetTopKeyword( string keyword )
        {
            if (keyword == "Quantity")
            {
                return "SUM(Quantity) as ProductQuantity, SUM(Quantity*UnitPrice) as ProductPrice";
            }
            else
            {
                return "SUM(Quantity*UnitPrice) as ProductPrice, SUM(Quantity) as ProductQuantity";
            }
        }

        private string GetKeyword( string keyword )
        {
            if (keyword == "Quantity")
            {
                return "SUM(Quantity) DESC";
            }
            else
            {
                return "SUM(Quantity*UnitPrice) DESC";
            }
        }

        private string GetTopNumber( string numberOfResult )
        {
            if (numberOfResult == "All")
            {
                return "";
            }
            else
            {
                return "TOP " + numberOfResult;
            }
        }

        private DataTable GetBestSellingReport(
            string topKeyword,
            string cultureID,
            string numberOfResult,
            string sortBy,
            string storeID,
            DateTime startOrderDate,
            DateTime endOrderDate,
            PeriodType period )
        {
            string from;
            string to;
            string storeFilter = "";
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( cultureID ) );
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( false ) );
            
            if (period == PeriodType.Last24Hours)
            {
                from = DateFilterHelper.CreateFromDateFor24HoursWithAnd(
                    startOrderDate, parameters, "from", "OrderDate" );
                to = DateFilterHelper.CreateToDateFor24HoursWithAnd(
                    endOrderDate, parameters, "to", "OrderDate" );
            }
            else
            {
                from = DateFilterHelper.CreateFromDateWithAnd(
                    startOrderDate, parameters, "from", "OrderDate" );
                to = DateFilterHelper.CreateToDateWithAnd(
                    endOrderDate, parameters, "to", "OrderDate" );
            }
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            string sort = "";
            if (numberOfResult != "All")
            {
                sort = "ORDER BY " + GetKeyword( topKeyword );
            }

            if (storeID != "0")
            {
                storeFilter = "AND Orders.StoreID = @StoreID2 ";
                parameters.Add( DataAccess.CreateParameterString( storeID ) );
            }
            string script = "SELECT ProductPrice, ProductQuantity, " +
                " Product.ProductID, PP.Price, L.Name, Product.Sku " +
                "FROM ((Product LEFT JOIN (SELECT [Name], ProductID FROM ProductLocale " +
                "WHERE CultureID = @CultureID ) as L on Product.ProductID = L.ProductID ) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "INNER JOIN (SELECT " + GetTopNumber( numberOfResult ) + " " + GetTopKeyword( topKeyword ) + " " +
                ", ProductID " +
                "FROM Orders, OrderItem " +
                "WHERE PaymentComplete = @PaymentComplete " +
                "AND Processed = @Processed And Cancelled = @Cancelled " +
                from +
                to +
                "AND Orders.OrderID = OrderItem.OrderID " +
                storeFilter +
                "GROUP BY ProductID " +
                sort +
                ") AS OI " +
                "ON Product.ProductID = OI.ProductID ";
            //+ sort;

            DataTable BestSellingReportTable = DataAccess.ExecuteSelect( script,
                DataAccess.ConvertArrayListToDbParameters( parameters ) );

            // For MS Access, we cannot use "TOP" keyword, so delete rows manually.
            if (numberOfResult != "All")
            {
                int numresult = Convert.ToInt32( numberOfResult );

                if (BestSellingReportTable.Rows.Count > numresult)
                {
                    int numberOfRemove = BestSellingReportTable.Rows.Count - numresult;
                    for (int i = 0; i < numberOfRemove; i++)
                        BestSellingReportTable.Rows.RemoveAt( numresult );
                }
            }

            BestSellingReportTable.DefaultView.Sort = sortBy;

            return BestSellingReportTable;

        }

        private DataTable GetBestSellingReport(
            string topKeyword,
            string cultureID,
            string numberOfResult,
            string sortBy,
            string storeID,
            DateTime startOrderDate,
            DateTime endOrderDate,
            int startIndex,
            int endIndex,
            PeriodType period,
            out int howManyItems )
        {
            DataTable full = GetBestSellingReport( topKeyword, cultureID, numberOfResult,
                sortBy, storeID, startOrderDate, endOrderDate, period );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["Product.ProductID"] };
            full.PrimaryKey = keys;

            DataView view = new DataView( full );
            view.Sort = sortBy;

            DataTable sortedTable = view.ToTable();

            return DataAccess.CopyTableRows( sortedTable, startIndex, endIndex );
        }

        private string ConvertToTopKeyword( BestSellReportType bestSellReportType )
        {
            string topKeyword = String.Empty;
            if (bestSellReportType == BestSellReportType.NumberOfProduct)
            {
                topKeyword = "Quantity";
            }
            else if (bestSellReportType == BestSellReportType.ProductTotalPrice)
            {
                topKeyword = "Total";
            }
            return topKeyword;
        }

        public DataTable GetBestSellingReportData(
            string sortBy,
            string cultureID,
            PeriodType period,
            BestSellReportType bestSellReportType,
            string numDisplay,
            string storeID,
            DateTime startOrderDate,
            DateTime endOrderDate,
            int startIndex,
            int endIndex,
            out int totalItems )
        {
            string topKeyword = ConvertToTopKeyword( bestSellReportType );

            DataTable table = GetBestSellingReport(
                topKeyword,
                cultureID,
                numDisplay,
                sortBy,
                storeID,
                startOrderDate,
                endOrderDate,
                startIndex,
                endIndex,
                period,
                out totalItems );

            return table;
        }

        public DataTable GetBestSellingReportData(
            string sortBy,
            string cultureID,
            PeriodType period,
            BestSellReportType bestSellReportType,
            string numDisplay,
            string storeID,
            DateTime startOrderDate,
            DateTime endOrderDate
            )
        {
            string topKeyword = ConvertToTopKeyword( bestSellReportType );

            DataTable table = GetBestSellingReport(
                topKeyword,
                cultureID,
                numDisplay,
                sortBy,
                storeID,
                startOrderDate,
                endOrderDate,
                period );

            return table;
        }
    }
}
